"use strict";
/*
    TODO:
    1. make sure that if requets rejected because auth cookie has expired,
        then delete saved cookie and throw the appropriate error
*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandExecutor = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const node_readline_1 = __importDefault(require("node:readline"));
const promises_1 = require("node:fs/promises");
const constants_1 = require("./lib/constants");
const types_1 = require("./lib/types");
const loader_1 = require("./lib/utils/loader");
const imaluum_1 = require("./imaluum");
const utils_1 = require("./lib/utils");
class CommandExecutor {
    constructor() {
        this.imaluum = null;
        this.errorCallback = null;
    }
    static async setup() {
        // create dump dir
        node_fs_1.default.mkdirSync(constants_1.DUMP_BASE_PATH, { recursive: true });
        const executor = new CommandExecutor();
        executor.imaluum = await imaluum_1.IMaluumPage.launch();
        return executor;
    }
    async execute(command, ...args) {
        if (command === types_1.CommandEnum.Authenticate) {
            await this.authenticate(...args);
        }
        else {
            try {
                // check if got cookie file
                // if yes, use `loginFromCookie`
                // else, throw error `unauthenticatd` or something
                await this.loginFromCookie();
            }
            catch (e) {
                // console.error(e);
                throw this.error(`${(0, utils_1.styleText)("You are not yet authenticated!", types_1.AnsiColourEnum.RED, types_1.AnsiTextStyleEnum.BOLD)} Please run the ${(0, utils_1.styleText)("authenticate", types_1.AnsiColourEnum.CYAN)} command first.`);
            }
            switch (command) {
                case types_1.CommandEnum.Result: {
                    await this.result(...args);
                    break;
                }
                case types_1.CommandEnum.Timetable: {
                    await this.timetable(...args);
                    break;
                }
                case types_1.CommandEnum.Test: {
                    await this._test(command, ...args);
                    break;
                }
                default:
                    throw new Error("UNKNOWN COMMAND!");
            }
        }
        await this.imaluum?.close();
    }
    setErrorCallback(fn) {
        this.errorCallback = fn;
    }
    error(message, errorOptions) {
        return this.errorCallback
            ? this.errorCallback(message, errorOptions)
            : new Error(message);
    }
    async loginFromCookie() {
        const str = await (0, utils_1.readFromFileSync)(constants_1.COOKIE_FILE_PATH);
        const cookeis = JSON.parse(str);
        await this.imaluum?.page?.setCookie(...cookeis);
        const homePage = "https://imaluum.iium.edu.my/home";
        await this.imaluum?.page?.goto(homePage);
        const currentUrl = this.imaluum?.page?.url();
        if (currentUrl !== homePage)
            throw new Error(`Wrong page. Expected ${homePage} but got ${currentUrl}.`);
    }
    async login() {
        const { username, password } = await this._getSavedCredentials();
        try {
            await this._loginWithUsername(username, password);
        }
        catch (e) {
            throw e;
        }
    }
    async _loginWithUsername(username, password) {
        const data = await this.imaluum?.login(username, password);
        // save cookies
        (0, utils_1.writeToFileSync)(constants_1.COOKIE_FILE_PATH, JSON.stringify(data?.cookies || ""));
    }
    async authenticate(...args) {
        try {
            const { username, password } = await this._authenticate({ options: args[0] });
            (0, utils_1.writeToFileSync)(constants_1.CREDENTIALS_FILE_PATH, `${username}\n${password}`);
            console.log(`🎉 ${(0, utils_1.styleText)("You are authenticated!", types_1.AnsiColourEnum.GREEN, types_1.AnsiTextStyleEnum.BOLD)}`);
        }
        catch (e) {
            // console.error(e);
            throw e;
        }
    }
    _authenticate({ options }) {
        return new Promise(async (resolve, reject) => {
            const credentials = {
                username: options.username,
                password: options.password,
            };
            try {
                if (options.interactive) {
                    const userInputs = await this._promptForUserCredentials();
                    credentials.username = userInputs[0];
                    credentials.password = userInputs[1];
                }
                if (!credentials.username)
                    throw new Error("Username is missing");
                if (!credentials.password)
                    throw new Error("Password is missing");
                const loader = new loader_1.Loader("Authenticating to i-Ma'luum");
                await loader.startAsyncTask(async () => await this._loginWithUsername(credentials.username, credentials.password));
                resolve(credentials);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async _promptForUserCredentials() {
        const rl = node_readline_1.default.createInterface({
            input: process.stdin,
            output: process.stdout,
        });
        const prompt = async (query) => await new Promise((resolve) => rl.question(query, resolve));
        try {
            const username = (await prompt(`${(0, utils_1.styleText)("# Username >> ", types_1.AnsiTextStyleEnum.BOLD)}`));
            const password = (await prompt(`${(0, utils_1.styleText)("# Password >> ", types_1.AnsiTextStyleEnum.BOLD)}`));
            await loader_1.TTYSync.clearAboveUntilSync(3);
            rl.close();
            return [username, password];
        }
        catch (error) {
            throw error;
        }
    }
    async result(...args) {
        if (this.imaluum?.page == null)
            throw new Error("i-Ma'luum page not running!");
        const resultArgs = {
            semester: args[0],
            year: args[1],
            options: args[2],
        };
        const loader = new loader_1.Loader(`Fetching exam result for Sem ${resultArgs.semester} ${resultArgs.year}`);
        const output = await loader.startAsyncTask(async () => {
            const resultPage = await this.imaluum?.goToSubPage("RESULT");
            return await this._result(resultPage, resultArgs);
        });
        this._displayTable(output);
    }
    _result(page, { semester, year, options }) {
        return new Promise(async (resolve, reject) => {
            if (semester.match(/^[123]$/) == null)
                throw new Error("Invalid <semester>. <semester> must be in the range of 1 >= semester <= 3.");
            if (year.match(/^([\d]*)\/([\d]*)*$/) == null)
                throw new Error("Invalid <year>. Hint: Make sure it is in the format XXXX/XXXX. (e.g. 2021/2022)");
            await page.selectTimetableOrResultDropDownMenuItem(semester, year);
            const tableStr = await page.extractTableElement();
            if (tableStr.length !== 0)
                resolve(tableStr);
            else
                reject();
        });
    }
    async timetable(...args) {
        if (this.imaluum?.page == null)
            throw new Error("i-Ma'luum page not running!");
        const timetableArgs = {
            semester: args[0],
            year: args[1],
            options: args[2],
        };
        const loader = new loader_1.Loader(`Fetching timetable for Sem ${timetableArgs.semester} ${timetableArgs.year}`);
        const output = await loader.startAsyncTask(async () => {
            const timetablePage = await this.imaluum?.goToSubPage("TIMETABLE");
            return await this._timetable(timetablePage, timetableArgs);
        });
        this._displayTable(output);
    }
    async _timetable(page, { semester, year, options }) {
        return new Promise(async (resolve, reject) => {
            if (semester.match(/^[123]$/) == null) {
                throw new Error("Invalid <semester>. <semester> must be in the range of 1 >= semester <= 3.");
            }
            if (year.match(/^([\d]*)\/([\d]*)*$/) == null) {
                throw new Error("Invalid <year>. Hint: Make sure it is in the format XXXX/XXXX. (e.g. 2021/2022)");
            }
            await page.selectTimetableOrResultDropDownMenuItem(semester, year);
            const tableStr = await page.extractTableElement();
            if (tableStr.length !== 0)
                resolve(tableStr);
            else
                reject();
        });
    }
    _displayTable(table) {
        console.log(table);
    }
    async _getSavedCredentials() {
        const values = [];
        try {
            const file = await (0, promises_1.open)(constants_1.CREDENTIALS_FILE_PATH, "as+");
            const fileStream = file.createReadStream({ encoding: "utf8" });
            // taken from https://nodejs.org/api/readline.html#readline_example_read_file_stream_line_by_line
            const rl = node_readline_1.default.createInterface({
                input: fileStream,
                crlfDelay: Infinity,
            });
            // Note: we use the crlfDelay option to recognize all instances of CR LF
            // ('\r\n') in input.txt as a single line break.
            for await (const line of rl) {
                // Each line in input.txt will be successively available here as `line`.
                // console.log(`Line from file: ${line}`);
                values.push(line);
            }
            return {
                username: values[0],
                password: values[1],
            };
        }
        catch (e) {
            throw e;
        }
    }
    async _test(command, ...args) {
        console.log("this is a test", command, args);
    }
}
exports.CommandExecutor = CommandExecutor;
