"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IMaluumSubPage = exports.IMaluumPage = void 0;
const puppeteer_1 = __importDefault(require("puppeteer"));
const table_1 = require("table");
const constants_1 = require("./lib/constants");
const utils_1 = require("./lib/utils");
class Page {
    constructor() {
        this.browser = null;
        this.page = null;
    }
    async _open() {
        this.browser = await puppeteer_1.default.launch({
            headless: true,
        });
        this.page = await this.browser.newPage();
    }
    async _close() {
        if (this.browser == null)
            throw new Error("Unable to close browser. No browser is running!");
        await this.browser.close();
    }
}
class IMaluumPage extends Page {
    // static async launch(): Promise<IMaluumPage> {
    // 	const page = await Page.launch();
    // }
    async close() {
        await this._close();
    }
    static async launch() {
        const page = new IMaluumPage();
        await page._open();
        return page;
    }
    async login(username, password) {
        if (!username || !password)
            throw new Error("User credentials missing!");
        return await this._loginToIMaluum(username, password);
    }
    // this function will always assume that `username` & `password` is not null,
    async _loginToIMaluum(username, password) {
        await this.goToLoginPage();
        await this.page?.$eval("#username", 
        // @ts-ignore
        (node, value) => (node.value = value), username);
        await this.page?.$eval("#password", 
        // @ts-ignore
        (node, value) => (node.value = value), password);
        const [response] = await Promise.all([
            this.page?.waitForNavigation({
                waitUntil: "networkidle0",
            }),
            this.page?.$eval("#fm1 input.btn-submit", (node) => {
                // @ts-ignore
                node.disabled = false;
                // @ts-ignore
                node.click();
            }),
        ]);
        if (response?.status() == 401) {
            const invalidStr = await this.page?.$eval("form#fm1 div.alert.alert-danger span", (elem) => elem.textContent);
            if (invalidStr == "Invalid credentials.")
                throw new Error("Invalid credentials");
            else
                throw new Error("Unable to proceed from login page");
        }
        // check that we have landed in the expected page (home page)
        const currentUrl = this.page?.url();
        if (currentUrl !== constants_1.IMALUUM_HOME_PAGE) {
            console.log("test");
            throw new Error(`Wrong page. Expected ${constants_1.IMALUUM_HOME_PAGE} but got ${currentUrl}.`);
        }
        const cookies = (await this.page?.cookies())?.filter((value) => {
            if (value.name == "XSRF-TOKEN" || value.name == "laravel_session")
                return value;
            else if (value.name == "MOD_AUTH_CAS") {
                value.expires = 10000000000;
                value.session = false;
                value.secure = false;
                return value;
            }
        });
        return {
            response,
            cookies: cookies || [],
        };
    }
    async goToLoginPage() {
        await this.page?.goto(constants_1.IMALUUM_LOGIN_PAGE, {});
    }
    // should only be used when on domain http://imaluum.iium.edu.my
    async goToSubPage(page) {
        if (this.page == null)
            throw new Error("Puppeteer page not running");
        const url = constants_1.IMALUUM_SUBPAGE_LINKS[page];
        await this.page.$eval(`a[href="${url}"]`, (elem) => {
            // @ts-expect-error
            elem.click();
        });
        await this.page.waitForNavigation();
        const currentUrl = this.page.url();
        if (currentUrl !== url)
            throw new Error(`Wrong page. Expected ${url} but got ${currentUrl}.`);
        return new IMaluumSubPage(this.page, page);
    }
}
exports.IMaluumPage = IMaluumPage;
class IMaluumSubPage {
    constructor(page, type) {
        this.page = page;
        this.type = type;
    }
    // `width` the extracted table width
    async extractTableElement() {
        const tableHTMLString = await this.page?.$eval("section.content table", (elem) => elem.outerHTML);
        if (this.type == "TIMETABLE")
            return (0, table_1.table)((0, utils_1.parseHTMLTableJson)(tableHTMLString));
        else {
            let tableJson = (0, utils_1.parseHTMLTableJson)(tableHTMLString);
            // remove last elem bcs not valid table elem
            // TODO: parse this last elem
            tableJson.pop();
            return (0, table_1.table)(tableJson);
        }
    }
    async selectTimetableOrResultDropDownMenuItem(semester, year) {
        const [elemHandles, elemTextContents] = await this._extractDropDownMenuItem();
        const selected = {
            fullStr: `Sem ${semester}, ${year}`,
            idx: -1,
        };
        elemTextContents.find((elem, idx) => elem === selected.fullStr ? (selected.idx = idx) : null);
        if (selected.idx === -1)
            throw new Error(`[ERROR] ${(0, utils_1.capitalize)(this.type)} for ${selected.fullStr} does not exist!`);
        // @ts-ignore
        elemHandles[selected.idx].evaluate((elem) => elem.click());
        await this.page.waitForNavigation();
    }
    async _extractDropDownMenuItem() {
        // make a check if dropdown menu elem exist
        const elemHandles = await this.page.$$("section.content ul.dropdown-menu a");
        const elemTextContent = await this.page.$$eval("section.content ul.dropdown-menu a", (elems) => elems.map((elem) => elem.textContent));
        return [elemHandles, elemTextContent];
    }
}
exports.IMaluumSubPage = IMaluumSubPage;
